@@ -1,8 +1,9 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
-// SPDX-License-Identifier: GPL-3.0+
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include <assert.h>
 #include <dwarf.h>
+#include <elfutils/libdwfl.h>
 #include <inttypes.h>
 #include <libelf.h>
 #include <stdlib.h>
@@ -14,6 +15,7 @@
 #include "dwarf_index.h"
 #include "error.h"
 #include "path.h"
+#include "platform.h"
 #include "siphash.h"
 #include "util.h"
 
@@ -107,8 +109,8 @@ drgn_dwarf_index_cu_buffer_error(struct binary_buffer *bb, const char *pos,
 {
 	struct drgn_dwarf_index_cu_buffer *buffer =
 		container_of(bb, struct drgn_dwarf_index_cu_buffer, bb);
-	return drgn_error_debug_info(buffer->cu->module, DRGN_SCN_DEBUG_INFO,
-				     pos, message);
+	return drgn_error_debug_info_scn(buffer->cu->module,
+					 DRGN_SCN_DEBUG_INFO, pos, message);
 }
 
 static void
@@ -116,7 +118,7 @@ drgn_dwarf_index_cu_buffer_init(struct drgn_dwarf_index_cu_buffer *buffer,
 				struct drgn_dwarf_index_cu *cu)
 {
 	binary_buffer_init(&buffer->bb, cu->buf, cu->len,
-			   cu->module->little_endian,
+			   drgn_platform_is_little_endian(&cu->module->platform),
 			   drgn_dwarf_index_cu_buffer_error);
 	buffer->cu = cu;
 }
@@ -127,8 +129,8 @@ DEFINE_VECTOR_FUNCTIONS(drgn_dwarf_index_cu_vector)
 struct drgn_dwarf_index_pending_die {
 	/* Index of compilation unit containing DIE. */
 	size_t cu;
-	/* Offset of DIE in .debug_info. */
-	size_t offset;
+	/* Address of DIE */
+	uintptr_t addr;
 };
 
 DEFINE_VECTOR_FUNCTIONS(drgn_dwarf_index_pending_die_vector)
@@ -322,7 +324,7 @@ read_abbrev_decl(struct drgn_debug_info_buffer *buffer,
 		} else if (name == DW_AT_name && should_index) {
 			switch (form) {
 			case DW_FORM_strp:
-				if (!cu->module->scns[DRGN_SCN_DEBUG_STR]) {
+				if (!cu->module->scn_data[DRGN_SCN_DEBUG_STR]) {
 					return binary_buffer_error(&buffer->bb,
 								   "DW_FORM_strp without .debug_str section");
 				}
@@ -338,7 +340,7 @@ read_abbrev_decl(struct drgn_debug_info_buffer *buffer,
 				break;
 			}
 		} else if (name == DW_AT_stmt_list &&
-			   cu->module->scns[DRGN_SCN_DEBUG_LINE]) {
+			   cu->module->scn_data[DRGN_SCN_DEBUG_LINE]) {
 			switch (form) {
 			case DW_FORM_data4:
 				insn = ATTRIB_STMT_LIST_LINEPTR4;
@@ -535,7 +537,7 @@ static struct drgn_error *read_abbrev_table(struct drgn_dwarf_index_cu *cu,
 	for (;;) {
 		struct drgn_error *err = read_abbrev_decl(&buffer, cu, &decls,
 							  &insns);
-		if (err && err->code == DRGN_ERROR_STOP) {
+		if (err == &drgn_stop) {
 			break;
 		} else if (err) {
 			uint8_vector_deinit(&insns);
@@ -574,7 +576,7 @@ static struct drgn_error *read_cu(struct drgn_dwarf_index_cu_buffer *buffer)
 			return err;
 	}
 	if (debug_abbrev_offset >
-	    buffer->cu->module->scns[DRGN_SCN_DEBUG_ABBREV]->d_size) {
+	    buffer->cu->module->scn_data[DRGN_SCN_DEBUG_ABBREV]->d_size) {
 		return binary_buffer_error(&buffer->bb,
 					   "debug_abbrev_offset is out of bounds");
 	}
@@ -743,12 +745,12 @@ out_directories:
 
 static struct drgn_error *
 index_specification(struct drgn_dwarf_index *dindex, uintptr_t declaration,
-		    struct drgn_debug_info_module *module, size_t offset)
+		    struct drgn_debug_info_module *module, uintptr_t addr)
 {
 	struct drgn_dwarf_index_specification entry = {
 		.declaration = declaration,
 		.module = module,
-		.offset = offset,
+		.addr = addr,
 	};
 	struct hash_pair hp =
 		drgn_dwarf_index_specification_map_hash(&declaration);
@@ -774,11 +776,11 @@ index_cu_first_pass(struct drgn_dwarf_index *dindex,
 {
 	struct drgn_error *err;
 	struct drgn_dwarf_index_cu *cu = buffer->cu;
-	Elf_Data *debug_info = cu->module->scns[DRGN_SCN_DEBUG_INFO];
+	Elf_Data *debug_info = cu->module->scn_data[DRGN_SCN_DEBUG_INFO];
 	const char *debug_info_buffer = debug_info->d_buf;
 	unsigned int depth = 0;
 	for (;;) {
-		size_t die_offset = buffer->bb.pos - debug_info_buffer;
+		size_t die_addr = (uintptr_t)buffer->bb.pos;
 
 		uint64_t code;
 		if ((err = binary_buffer_next_uleb128(&buffer->bb, &code)))
@@ -958,7 +960,7 @@ skip:
 		if (depth == 0) {
 			if (stmt_list_ptr) {
 				if (stmt_list >
-				    cu->module->scns[DRGN_SCN_DEBUG_LINE]->d_size) {
+				    cu->module->scn_data[DRGN_SCN_DEBUG_LINE]->d_size) {
 					return binary_buffer_error_at(&buffer->bb,
 								      stmt_list_ptr,
 								      "DW_AT_stmt_list is out of bounds");
@@ -978,7 +980,7 @@ skip:
 			 */
 			if (!declaration &&
 			    (err = index_specification(dindex, specification,
-						       cu->module, die_offset)))
+						       cu->module, die_addr)))
 				return err;
 		}
 
@@ -1062,9 +1064,11 @@ err:
 	drgn_dwarf_index_update_cancel(state, err);
 }
 
-static bool find_definition(struct drgn_dwarf_index *dindex, uintptr_t die_addr,
-			    struct drgn_debug_info_module **module_ret,
-			    size_t *offset_ret)
+bool
+drgn_dwarf_index_find_definition(struct drgn_dwarf_index *dindex,
+				 uintptr_t die_addr,
+				 struct drgn_debug_info_module **module_ret,
+				 uintptr_t *addr_ret)
 {
 	struct drgn_dwarf_index_specification_map_iterator it =
 		drgn_dwarf_index_specification_map_search(&dindex->specifications,
@@ -1072,7 +1076,7 @@ static bool find_definition(struct drgn_dwarf_index *dindex, uintptr_t die_addr,
 	if (!it.entry)
 		return false;
 	*module_ret = it.entry->module;
-	*offset_ret = it.entry->offset;
+	*addr_ret = it.entry->addr;
 	return true;
 }
 
@@ -1080,7 +1084,7 @@ static bool append_die_entry(struct drgn_dwarf_index *dindex,
 			     struct drgn_dwarf_index_shard *shard, uint8_t tag,
 			     uint64_t file_name_hash,
 			     struct drgn_debug_info_module *module,
-			     size_t offset)
+			     uintptr_t addr)
 {
 	if (shard->dies.size == UINT32_MAX)
 		return false;
@@ -1101,7 +1105,7 @@ static bool append_die_entry(struct drgn_dwarf_index *dindex,
 		die->file_name_hash = file_name_hash;
 	}
 	die->module = module;
-	die->offset = offset;
+	die->addr = addr;
 
 	return true;
 }
@@ -1111,7 +1115,7 @@ static struct drgn_error *index_die(struct drgn_dwarf_index_namespace *ns,
 				    const char *name, uint8_t tag,
 				    uint64_t file_name_hash,
 				    struct drgn_debug_info_module *module,
-				    size_t offset)
+				    uintptr_t addr)
 {
 	struct drgn_error *err;
 	struct drgn_dwarf_index_die_map_entry entry = {
@@ -1133,7 +1137,7 @@ static struct drgn_error *index_die(struct drgn_dwarf_index_namespace *ns,
 						    hp);
 	if (!it.entry) {
 		if (!append_die_entry(ns->dindex, shard, tag, file_name_hash,
-				      module, offset)) {
+				      module, addr)) {
 			err = &drgn_enomem;
 			goto err;
 		}
@@ -1162,7 +1166,7 @@ static struct drgn_error *index_die(struct drgn_dwarf_index_namespace *ns,
 
 	index = die - shard->dies.data;
 	if (!append_die_entry(ns->dindex, shard, tag, file_name_hash, module,
-			      offset)) {
+			      addr)) {
 		err = &drgn_enomem;
 		goto err;
 	}
@@ -1177,7 +1181,7 @@ out:
 			goto err;
 		}
 		pending->cu = cu - ns->dindex->cus.data;
-		pending->offset = offset;
+		pending->addr = addr;
 	}
 	err = NULL;
 err:
@@ -1192,14 +1196,12 @@ index_cu_second_pass(struct drgn_dwarf_index_namespace *ns,
 {
 	struct drgn_error *err;
 	struct drgn_dwarf_index_cu *cu = buffer->cu;
-	Elf_Data *debug_info = cu->module->scns[DRGN_SCN_DEBUG_INFO];
-	const char *debug_info_buffer = debug_info->d_buf;
-	Elf_Data *debug_str = cu->module->scns[DRGN_SCN_DEBUG_STR];
+	Elf_Data *debug_str = cu->module->scn_data[DRGN_SCN_DEBUG_STR];
 	unsigned int depth = 0;
 	uint8_t depth1_tag = 0;
-	size_t depth1_offset = 0;
+	size_t depth1_addr = 0;
 	for (;;) {
-		size_t die_offset = buffer->bb.pos - debug_info_buffer;
+		size_t die_addr = (uintptr_t)buffer->bb.pos;
 
 		uint64_t code;
 		if ((err = binary_buffer_next_uleb128(&buffer->bb, &code)))
@@ -1389,7 +1391,7 @@ skip:
 		uint8_t tag = insn & DIE_FLAG_TAG_MASK;
 		if (depth == 1) {
 			depth1_tag = tag;
-			depth1_offset = die_offset;
+			depth1_addr = die_addr;
 		}
 		if (depth == (tag == DW_TAG_enumerator ? 2 : 1) && name &&
 		    !specification) {
@@ -1405,13 +1407,13 @@ skip:
 				 * be declared in C/C++, so we don't check for
 				 * that.
 				 */
-				die_offset = depth1_offset;
+				die_addr = depth1_addr;
 			} else if (declaration &&
-				   !find_definition(ns->dindex,
-						    (uintptr_t)debug_info_buffer +
-						    die_offset,
-						    &module, &die_offset)) {
-					goto next;
+				   !drgn_dwarf_index_find_definition(ns->dindex,
+								     die_addr,
+								     &module,
+								     &die_addr)) {
+				goto next;
 			}
 
 			uint64_t file_name_hash;
@@ -1427,7 +1429,7 @@ skip:
 				file_name_hash = 0;
 			}
 			if ((err = index_die(ns, cu, name, tag, file_name_hash,
-					     module, die_offset)))
+					     module, die_addr)))
 				return err;
 		}
 
@@ -1563,7 +1565,7 @@ static struct drgn_error *index_namespace(struct drgn_dwarf_index_namespace *ns)
 				&ns->dindex->cus.data[pending->cu];
 			struct drgn_dwarf_index_cu_buffer buffer;
 			drgn_dwarf_index_cu_buffer_init(&buffer, cu);
-			buffer.bb.pos += pending->offset;
+			buffer.bb.pos = (char *)pending->addr;
 			struct drgn_error *cu_err =
 				index_cu_second_pass(ns, &buffer);
 			if (cu_err) {
@@ -1688,7 +1690,9 @@ struct drgn_error *drgn_dwarf_index_get_die(struct drgn_dwarf_index_die *die,
 	Dwarf *dwarf = dwfl_module_getdwarf(die->module->dwfl_module, &bias);
 	if (!dwarf)
 		return drgn_error_libdwfl();
-	if (!dwarf_offdie(dwarf, die->offset, die_ret))
+	uintptr_t start =
+		(uintptr_t)die->module->scn_data[DRGN_SCN_DEBUG_INFO]->d_buf;
+	if (!dwarf_offdie(dwarf, die->addr - start, die_ret))
 		return drgn_error_libdw();
 	return NULL;
 }
