@@ -1,5 +1,5 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
-// SPDX-License-Identifier: GPL-3.0+
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include <dirent.h>
 #include <elf.h>
@@ -316,8 +316,7 @@ struct drgn_error *linux_kernel_object_find(const char *name, size_t name_len,
 			qualified_type.qualifiers = 0;
 			return drgn_object_set_from_buffer(ret, qualified_type,
 							   prog->vmcoreinfo.osrelease,
-							   len + 1, 0, 0,
-							   DRGN_PROGRAM_ENDIAN);
+							   len + 1, 0, 0);
 		} else if (name_len == strlen("vmemmap") &&
 			   memcmp(name, "vmemmap", name_len) == 0) {
 			if (prog->vmemmap.kind == DRGN_OBJECT_ABSENT) {
@@ -453,7 +452,7 @@ kernel_module_iterator_next_live(struct kernel_module_iterator *it)
  * destroyed.
  *
  * @return @c NULL on success, non-@c NULL on error. In particular, when there
- * are no more modules, a @ref DRGN_ERROR_STOP error is returned.
+ * are no more modules, returns &@ref drgn_stop.
  */
 static struct drgn_error *
 kernel_module_iterator_next(struct kernel_module_iterator *it)
@@ -909,7 +908,7 @@ static struct drgn_error *identify_kernel_elf(Elf *elf,
 
 		scnname = elf_strptr(elf, shstrndx, shdr->sh_name);
 		if (!scnname)
-			continue;
+			return drgn_error_libelf();
 		if (strcmp(scnname, ".gnu.linkonce.this_module") == 0)
 			*this_module_scn_ret = scn;
 		else if (strcmp(scnname, ".modinfo") == 0)
@@ -1013,13 +1012,10 @@ cache_kernel_module_sections(struct kernel_module_iterator *kmod_it, Elf *elf,
 			.key = elf_strptr(elf, shstrndx, shdr->sh_name),
 			.value = scn,
 		};
-		/*
-		 * .init sections are freed once the module is initialized, but
-		 * they remain in the section list. Ignore them so we don't get
-		 * confused if the address gets reused for another module.
-		 */
-		if (!entry.key || strstartswith(entry.key, ".init"))
-			continue;
+		if (!entry.key) {
+			err = drgn_error_libelf();
+			goto out_scn_map;
+		}
 
 		if (elf_scn_name_map_insert(&scn_map, &entry, NULL) == -1) {
 			err = &drgn_enomem;
@@ -1051,19 +1047,28 @@ cache_kernel_module_sections(struct kernel_module_iterator *kmod_it, Elf *elf,
 				err = drgn_error_libelf();
 				break;
 			}
-			uint64_t section_end;
-			if (__builtin_add_overflow(address, shdr->sh_size,
-						   &section_end))
-				section_end = UINT64_MAX;
-			if (address < section_end) {
-				if (address < start)
-					start = address;
-				if (section_end > end)
-					end = section_end;
+			/*
+			 * .init sections are freed once the module is
+			 * initialized, but they remain in the section list.
+			 * Ignore them for the purpose of determining the
+			 * module's address range.
+			 */
+			if (!strstartswith(name, ".init")) {
+				uint64_t section_end;
+				if (__builtin_add_overflow(address,
+							   shdr->sh_size,
+							   &section_end))
+					section_end = UINT64_MAX;
+				if (address < section_end) {
+					if (address < start)
+						start = address;
+					if (section_end > end)
+						end = section_end;
+				}
 			}
 		}
 	}
-	if (err && err->code != DRGN_ERROR_STOP)
+	if (err && err != &drgn_stop)
 		goto out_section_it;
 	err = NULL;
 	if (start >= end)
@@ -1233,7 +1238,7 @@ kernel_module_iterator_error:
 	}
 	for (;;) {
 		err = kernel_module_iterator_next(&kmod_it);
-		if (err && err->code == DRGN_ERROR_STOP) {
+		if (err == &drgn_stop) {
 			err = NULL;
 			break;
 		} else if (err) {
