@@ -1,11 +1,12 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
-// SPDX-License-Identifier: GPL-3.0+
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include <assert.h>
 #include <byteswap.h>
 #include <dwarf.h>
 #include <elf.h>
 #include <elfutils/libdw.h>
+#include <elfutils/version.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <gelf.h>
@@ -33,8 +34,10 @@ DEFINE_HASH_TABLE_FUNCTIONS(drgn_prstatus_map, int_key_hash_pair, scalar_key_eq)
 
 static Elf_Type note_header_type(GElf_Phdr *phdr)
 {
+#if _ELFUTILS_PREREQ(0, 175)
 	if (phdr->p_align == 8)
 		return ELF_T_NHDR8;
+#endif
 	return ELF_T_NHDR;
 }
 
@@ -75,6 +78,8 @@ void drgn_program_init(struct drgn_program *prog,
 	prog->core_fd = -1;
 	if (platform)
 		drgn_program_set_platform(prog, platform);
+	char *env = getenv("DRGN_PREFER_ORC_UNWINDER");
+	prog->prefer_orc_unwinder = env && atoi(env);
 	drgn_object_init(&prog->page_offset, prog);
 	drgn_object_init(&prog->vmemmap, prog);
 }
