@@ -1,10 +1,11 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
-// SPDX-License-Identifier: GPL-3.0+
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include <stdarg.h>
 
 #include "drgnpy.h"
 #include "../lazy_object.h"
+#include "../platform.h"
 #include "../program.h"
 #include "../type.h"
 #include "../util.h"
@@ -143,6 +144,26 @@ static PyObject *DrgnType_get_is_signed(DrgnType *self)
 	Py_RETURN_BOOL(drgn_type_is_signed(self->type));
 }
 
+/*
+ * This returns one of two static strings, so it doesn't need the attribute
+ * cache.
+ */
+static PyObject *DrgnType_get_byteorder(DrgnType *self, void *arg)
+{
+	if (!drgn_type_has_little_endian(self->type)) {
+		return PyErr_Format(PyExc_AttributeError,
+				    "%s type does not have a byte order",
+				    drgn_type_kind_str(self->type));
+	}
+	_Py_IDENTIFIER(little);
+	_Py_IDENTIFIER(big);
+	PyObject *ret =
+		_PyUnicode_FromId(drgn_type_little_endian(self->type) ?
+				  &PyId_little : &PyId_big);
+	Py_XINCREF(ret);
+	return ret;
+}
+
 static PyObject *DrgnType_get_type(DrgnType *self)
 {
 	if (!drgn_type_has_type(self->type)) {
@@ -446,6 +467,8 @@ static PyGetSetDef DrgnType_getset[] = {
 	 &DrgnType_attr_length},
 	{"is_signed", (getter)DrgnType_getter, NULL, drgn_Type_is_signed_DOC,
 	 &DrgnType_attr_is_signed},
+	{"byteorder", (getter)DrgnType_get_byteorder, NULL,
+	 drgn_Type_byteorder_DOC},
 	{"type", (getter)DrgnType_getter, NULL, drgn_Type_type_DOC,
 	 &DrgnType_attr_type},
 	{"members", (getter)DrgnType_getter, NULL, drgn_Type_members_DOC,
@@ -527,13 +550,12 @@ static int append_field(PyObject *parts, bool *first, const char *format, ...)
 
 static PyObject *DrgnType_repr(DrgnType *self)
 {
-	PyObject *parts, *ret = NULL;
-	bool first = true;
-
-	parts = PyList_New(0);
+	PyObject *parts = PyList_New(0);
 	if (!parts)
 		return NULL;
 
+	PyObject *ret = NULL;
+	bool first = true;
 	if (append_format(parts, "prog.%s_type(",
 			  drgn_type_kind_str(self->type)) == -1)
 		goto out;
@@ -549,24 +571,24 @@ static PyObject *DrgnType_repr(DrgnType *self)
 		goto out;
 	if (append_member(parts, self, &first, type) == -1)
 		goto out;
-	if (drgn_type_kind(self->type) == DRGN_TYPE_POINTER) {
-		bool print_size;
-		if (drgn_type_program(self->type)->has_platform) {
-			uint8_t word_size;
-			struct drgn_error *err =
-				drgn_program_word_size(drgn_type_program(self->type),
-						       &word_size);
-			if (err) {
-				set_drgn_error(err);
-				goto out;
-			}
-			print_size = drgn_type_size(self->type) != word_size;
-		} else {
-			print_size = true;
-		}
-		if (print_size &&
-		    append_member(parts, self, &first, size) == -1)
+	if (drgn_type_kind(self->type) == DRGN_TYPE_POINTER &&
+	    (!drgn_type_program(self->type)->has_platform ||
+	     drgn_type_size(self->type) !=
+	     drgn_platform_address_size(&drgn_type_program(self->type)->platform)) &&
+	    append_member(parts, self, &first, size) == -1)
+		goto out;
+	if (drgn_type_has_little_endian(self->type) &&
+	    (!drgn_type_program(self->type)->has_platform ||
+	     drgn_type_little_endian(self->type) !=
+	     drgn_platform_is_little_endian(&drgn_type_program(self->type)->platform))) {
+		PyObject *obj = DrgnType_get_byteorder(self, NULL);
+		if (!obj)
 			goto out;
+		if (append_field(parts, &first, "byteorder=%R", obj) == -1) {
+			Py_DECREF(obj);
+			goto out;
+		}
+		Py_DECREF(obj);
 	}
 	if (append_member(parts, self, &first, length) == -1)
 		goto out;
@@ -583,12 +605,10 @@ static PyObject *DrgnType_repr(DrgnType *self)
 	    append_member(parts, self, &first, template_parameters) == -1)
 		goto out;
 	if (self->qualifiers) {
-		PyObject *obj;
-
-		obj = DrgnType_getter(self, &DrgnType_attr_qualifiers);
+		PyObject *obj = DrgnType_getter(self,
+						&DrgnType_attr_qualifiers);
 		if (!obj)
 			goto out;
-
 		if (append_field(parts, &first, "qualifiers=%R", obj) == -1) {
 			Py_DECREF(obj);
 			goto out;
@@ -956,7 +976,8 @@ static int append_lazy_object_repr(PyObject *parts, LazyObject *self)
 	DrgnObject *object = LazyObject_get_borrowed(self);
 	if (!object)
 		return -1;
-	if (object->obj.kind == DRGN_OBJECT_ABSENT) {
+	if (object->obj.kind == DRGN_OBJECT_ABSENT &&
+	    !object->obj.is_bit_field) {
 		char *type_name;
 		err = drgn_format_type_name(drgn_object_qualified_type(&object->obj),
 					    &type_name);
@@ -1353,19 +1374,57 @@ DrgnType *Program_void_type(Program *self, PyObject *args, PyObject *kwds)
 	return (DrgnType *)DrgnType_wrap(qualified_type);
 }
 
+struct byteorder_arg {
+	bool allow_none;
+	bool is_none;
+	enum drgn_byte_order value;
+};
+
+static int byteorder_converter(PyObject *o, void *p)
+{
+	struct byteorder_arg *arg = p;
+
+	arg->is_none = o == Py_None;
+	if (arg->allow_none && o == Py_None)
+		return 1;
+
+	if (PyUnicode_Check(o)) {
+		const char *s = PyUnicode_AsUTF8(o);
+		if (strcmp(s, "little") == 0) {
+			arg->value = DRGN_LITTLE_ENDIAN;
+			return 1;
+		} else if (strcmp(s, "big") == 0) {
+			arg->value = DRGN_BIG_ENDIAN;
+			return 1;
+		}
+	}
+	PyErr_Format(PyExc_ValueError,
+		     "expected 'little'%s 'big'%s for byteorder",
+		     arg->allow_none ? "," : " or",
+		     arg->allow_none ? ", or None" : "");
+	return 0;
+}
+
 DrgnType *Program_int_type(Program *self, PyObject *args, PyObject *kwds)
 {
 	static char *keywords[] = {
-		"name", "size", "is_signed", "qualifiers", "language", NULL
+		"name", "size", "is_signed", "byteorder", "qualifiers",
+		"language", NULL
 	};
 	PyObject *name_obj;
 	struct index_arg size = {};
 	int is_signed;
+	struct byteorder_arg byteorder = {
+		.allow_none = true,
+		.is_none = true,
+		.value = DRGN_PROGRAM_ENDIAN,
+	};
 	enum drgn_qualifiers qualifiers = 0;
 	const struct drgn_language *language = NULL;
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&p|$O&O&:int_type",
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&p|O&$O&O&:int_type",
 					 keywords, &PyUnicode_Type, &name_obj,
 					 index_converter, &size, &is_signed,
+					 byteorder_converter, &byteorder,
 					 qualifiers_converter, &qualifiers,
 					 language_converter, &language))
 		return NULL;
@@ -1380,7 +1439,7 @@ DrgnType *Program_int_type(Program *self, PyObject *args, PyObject *kwds)
 	struct drgn_qualified_type qualified_type;
 	struct drgn_error *err = drgn_int_type_create(&self->prog, name,
 						      size.uvalue, is_signed,
-						      language,
+						      byteorder.value, language,
 						      &qualified_type.type);
 	if (err)
 		return set_drgn_error(err);
@@ -1406,15 +1465,21 @@ DrgnType *Program_int_type(Program *self, PyObject *args, PyObject *kwds)
 DrgnType *Program_bool_type(Program *self, PyObject *args, PyObject *kwds)
 {
 	static char *keywords[] = {
-		"name", "size", "qualifiers", "language", NULL
+		"name", "size", "byteorder", "qualifiers", "language", NULL
 	};
 	PyObject *name_obj;
 	struct index_arg size = {};
+	struct byteorder_arg byteorder = {
+		.allow_none = true,
+		.is_none = true,
+		.value = DRGN_PROGRAM_ENDIAN,
+	};
 	enum drgn_qualifiers qualifiers = 0;
 	const struct drgn_language *language = NULL;
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&|$O&O&:bool_type",
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&|O&$O&O&:bool_type",
 					 keywords, &PyUnicode_Type, &name_obj,
 					 index_converter, &size,
+					 byteorder_converter, &byteorder,
 					 qualifiers_converter, &qualifiers,
 					 language_converter, &language))
 		return NULL;
@@ -1428,7 +1493,9 @@ DrgnType *Program_bool_type(Program *self, PyObject *args, PyObject *kwds)
 
 	struct drgn_qualified_type qualified_type;
 	struct drgn_error *err = drgn_bool_type_create(&self->prog, name,
-						       size.uvalue, language,
+						       size.uvalue,
+						       byteorder.value,
+						       language,
 						       &qualified_type.type);
 	if (err)
 		return set_drgn_error(err);
@@ -1454,16 +1521,22 @@ DrgnType *Program_bool_type(Program *self, PyObject *args, PyObject *kwds)
 DrgnType *Program_float_type(Program *self, PyObject *args, PyObject *kwds)
 {
 	static char *keywords[] = {
-		"name", "size", "qualifiers", "language", NULL
+		"name", "size", "byteorder", "qualifiers", "language", NULL
 	};
 	PyObject *name_obj;
 	struct index_arg size = {};
+	struct byteorder_arg byteorder = {
+		.allow_none = true,
+		.is_none = true,
+		.value = DRGN_PROGRAM_ENDIAN,
+	};
 	enum drgn_qualifiers qualifiers = 0;
 	const struct drgn_language *language = NULL;
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&|$O&O&:float_type",
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!O&|O&$O&O&:float_type",
 					 keywords, &PyUnicode_Type, &name_obj,
 					 index_converter, &size,
+					 byteorder_converter, &byteorder,
 					 qualifiers_converter, &qualifiers,
 					 language_converter, &language))
 		return NULL;
@@ -1477,7 +1550,9 @@ DrgnType *Program_float_type(Program *self, PyObject *args, PyObject *kwds)
 
 	struct drgn_qualified_type qualified_type;
 	struct drgn_error *err = drgn_float_type_create(&self->prog, name,
-							size.uvalue, language,
+							size.uvalue,
+							byteorder.value,
+							language,
 							&qualified_type.type);
 	if (err)
 		return set_drgn_error(err);
@@ -1500,72 +1575,6 @@ DrgnType *Program_float_type(Program *self, PyObject *args, PyObject *kwds)
 	return type_obj;
 }
 
-DrgnType *Program_complex_type(Program *self, PyObject *args, PyObject *kwds)
-{
-	static char *keywords[] = {
-		"name", "size", "type", "qualifiers", "language", NULL
-	};
-	PyObject *name_obj;
-	struct index_arg size = {};
-	DrgnType *real_type_obj;
-	enum drgn_qualifiers qualifiers = 0;
-	const struct drgn_language *language = NULL;
-	if (!PyArg_ParseTupleAndKeywords(args, kwds,
-					 "O!O&O!|$O&O&:complex_type", keywords,
-					 &PyUnicode_Type, &name_obj,
-					 index_converter, &size, &DrgnType_type,
-					 &real_type_obj, qualifiers_converter,
-					 &qualifiers, language_converter,
-					 &language))
-		return NULL;
-
-	const char *name = PyUnicode_AsUTF8(name_obj);
-	if (!name)
-		return NULL;
-
-	struct drgn_type *real_type = real_type_obj->type;
-	if (drgn_type_kind(real_type) != DRGN_TYPE_FLOAT &&
-	    drgn_type_kind(real_type) != DRGN_TYPE_INT) {
-		PyErr_SetString(PyExc_ValueError,
-				"complex_type() real type must be floating-point or integer type");
-		return NULL;
-	}
-	if (real_type_obj->qualifiers) {
-		PyErr_SetString(PyExc_ValueError,
-				"complex_type() real type must be unqualified");
-		return NULL;
-	}
-
-	if (!Program_hold_reserve(self, 1))
-		return NULL;
-
-	struct drgn_qualified_type qualified_type;
-	struct drgn_error *err = drgn_complex_type_create(&self->prog, name,
-							  size.uvalue,
-							  real_type, language,
-							  &qualified_type.type);
-	if (err)
-		return set_drgn_error(err);
-
-	if (drgn_type_name(qualified_type.type) == name)
-		Program_hold_object(self, name_obj);
-
-	qualified_type.qualifiers = qualifiers;
-	DrgnType *type_obj = (DrgnType *)DrgnType_wrap(qualified_type);
-	if (!type_obj)
-		return NULL;
-
-	if (_PyDict_SetItemId(type_obj->attr_cache, &DrgnType_attr_type.id,
-			      (PyObject *)real_type_obj) == -1 ||
-	    _PyDict_SetItemId(type_obj->attr_cache, &DrgnType_attr_name.id,
-			      name_obj) == -1) {
-		Py_DECREF(type_obj);
-		return NULL;
-	}
-
-	return type_obj;
-}
-
 static struct drgn_error *py_lazy_object_thunk_fn(struct drgn_object *res,
 						  void *arg)
 {
@@ -2084,33 +2093,40 @@ DrgnType *Program_typedef_type(Program *self, PyObject *args, PyObject *kwds)
 DrgnType *Program_pointer_type(Program *self, PyObject *args, PyObject *kwds)
 {
 	static char *keywords[] = {
-		"type", "size", "qualifiers", "language", NULL
+		"type", "size", "byteorder", "qualifiers", "language", NULL
 	};
 	DrgnType *referenced_type_obj;
 	struct index_arg size = { .allow_none = true, .is_none = true };
+	struct byteorder_arg byteorder = {
+		.allow_none = true,
+		.is_none = true,
+		.value = DRGN_PROGRAM_ENDIAN,
+	};
 	enum drgn_qualifiers qualifiers = 0;
 	const struct drgn_language *language = NULL;
-	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|O&$O&O&:pointer_type",
+	if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|O&O&$O&O&:pointer_type",
 					 keywords, &DrgnType_type,
 					 &referenced_type_obj, index_converter,
-					 &size, qualifiers_converter,
-					 &qualifiers, language_converter,
-					 &language))
+					 &size, byteorder_converter, &byteorder,
+					 qualifiers_converter, &qualifiers,
+					 language_converter, &language))
 		return NULL;
 
 	if (size.is_none) {
-		uint8_t word_size;
-		struct drgn_error *err = drgn_program_word_size(&self->prog,
-								&word_size);
+		uint8_t address_size;
+		struct drgn_error *err =
+			drgn_program_address_size(&self->prog, &address_size);
 		if (err)
 			return set_drgn_error(err);
-		size.uvalue = word_size;
+		size.uvalue = address_size;
 	}
 
 	struct drgn_qualified_type qualified_type;
 	struct drgn_error *err = drgn_pointer_type_create(&self->prog,
 							  DrgnType_unwrap(referenced_type_obj),
-							  size.uvalue, language,
+							  size.uvalue,
+							  byteorder.value,
+							  language,
 							  &qualified_type.type);
 	if (err)
 		return set_drgn_error(err);
