@@ -1,12 +1,10 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
-// SPDX-License-Identifier: GPL-3.0+
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #include <assert.h>
 #include <endian.h>
-#include <string.h>
 
 #include "serialize.h"
-#include "minmax.h"
 
 void serialize_bits(void *buf, uint64_t bit_offset, uint64_t uvalue,
 		    uint8_t bit_size, bool little_endian)
