@@ -1,5 +1,5 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
-// SPDX-License-Identifier: GPL-3.0+
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 /**
  * @file
@@ -14,6 +14,9 @@
 
 #include <stdbool.h>
 #include <stdint.h>
+#include <string.h>
+
+#include "minmax.h"
 
 /**
  * @ingroup Internals
@@ -25,34 +28,49 @@
  * @{
  */
 
-/** Copy the @p bit_size%th bit to the more-significant bits in an integer. */
-static inline int64_t sign_extend(int64_t svalue, uint64_t bit_size)
-{
-	if (bit_size < 64) {
-		int64_t mask;
-
-		mask = INT64_C(1) << (bit_size - 1);
-		svalue = (svalue ^ mask) - mask;
-	}
-	return svalue;
-}
-
-/** Truncate a signed integer to @p bit_size bits. */
+/** Truncate a signed integer to @p bit_size bits with sign extension. */
 static inline int64_t truncate_signed(int64_t svalue, uint64_t bit_size)
 {
-	if (bit_size < 64) {
-		/* INT64_C(1) << 63 is undefined, so this must be UINT64_C. */
-		svalue &= (UINT64_C(1) << bit_size) - 1;
-	}
-	return sign_extend(svalue, bit_size);
+	return (int64_t)((uint64_t)svalue << (64 - bit_size)) >> (64 - bit_size);
 }
 
 /** Truncate an unsigned integer to @p bit_size bits. */
 static inline uint64_t truncate_unsigned(uint64_t uvalue, uint64_t bit_size)
 {
-	if (bit_size < 64)
-		uvalue &= (UINT64_C(1) << bit_size) - 1;
-	return uvalue;
+	return uvalue << (64 - bit_size) >> (64 - bit_size);
+}
+
+/**
+ * Copy the @p src_size least-significant bytes from @p src to the @p dst_size
+ * least-significant bytes of @p dst.
+ *
+ * If `src_size > dst_size`, the extra bytes are discarded. If `src_size <
+ * dst_size`, the extra bytes are zero-filled.
+ */
+static inline void copy_lsbytes(void *dst, size_t dst_size,
+				bool dst_little_endian, const void *src,
+				size_t src_size, bool src_little_endian)
+{
+	char *d = dst;
+	const char *s = src;
+	size_t size = min(dst_size, src_size);
+	if (dst_little_endian) {
+		if (src_little_endian) {
+			memcpy(d, s, size);
+		} else {
+			for (size_t i = 0; i < size; i++)
+				d[i] = s[src_size - 1 - i];
+		}
+		memset(d + size, 0, dst_size - size);
+	} else {
+		memset(d, 0, dst_size - size);
+		if (src_little_endian) {
+			for (size_t i = dst_size - size; i < size; i++)
+				d[i] = s[dst_size - 1 - i];
+		} else {
+			memcpy(d + dst_size - size, s + src_size - size, size);
+		}
+	}
 }
 
 /**
