@@ -1,5 +1,5 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
-// SPDX-License-Identifier: GPL-3.0+
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 /**
  * @file
@@ -12,6 +12,8 @@
 #ifndef DRGN_STACK_TRACE_H
 #define DRGN_STACK_TRACE_H
 
+#include <stddef.h>
+
 /**
  * @ingroup Internals
  *
@@ -24,15 +26,14 @@
  * @{
  */
 
+struct drgn_stack_frame {
+	struct drgn_register_state *regs;
+};
+
 struct drgn_stack_trace {
 	struct drgn_program *prog;
-	union {
-		Dwfl_Thread *thread;
-		/* Used during creation. */
-		size_t capacity;
-	};
 	size_t num_frames;
-	Dwfl_Frame *frames[];
+	struct drgn_stack_frame frames[];
 };
 
 /** @} */
