@@ -1,6 +1,7 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
-// SPDX-License-Identifier: GPL-3.0+
+// SPDX-License-Identifier: GPL-3.0-or-later
 
+#include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 
@@ -17,7 +18,6 @@ const char * const drgn_type_kind_spelling[] = {
 	[DRGN_TYPE_INT] = "int",
 	[DRGN_TYPE_BOOL] = "bool",
 	[DRGN_TYPE_FLOAT] = "float",
-	[DRGN_TYPE_COMPLEX] = "complex",
 	[DRGN_TYPE_STRUCT] = "struct",
 	[DRGN_TYPE_UNION] = "union",
 	[DRGN_TYPE_CLASS] = "class",
@@ -263,6 +263,8 @@ drgn_type_dedupe_hash_pair(struct drgn_type * const *entry)
 		hash = hash_combine(hash, drgn_type_size(type));
 	if (drgn_type_has_is_signed(type))
 		hash = hash_combine(hash, drgn_type_is_signed(type));
+	if (drgn_type_has_little_endian(type))
+		hash = hash_combine(hash, drgn_type_little_endian(type));
 	const char *tag;
 	if (drgn_type_has_tag(type) && (tag = drgn_type_tag(type)))
 		hash = hash_combine(hash, hash_c_string(tag));
@@ -297,6 +299,9 @@ static bool drgn_type_dedupe_eq(struct drgn_type * const *entry_a,
 	if (drgn_type_has_is_signed(a) &&
 	    drgn_type_is_signed(a) != drgn_type_is_signed(b))
 		return false;
+	if (drgn_type_has_little_endian(a) &&
+	    drgn_type_little_endian(a) != drgn_type_little_endian(b))
+		return false;
 	if (drgn_type_has_tag(a)) {
 		const char *tag_a = drgn_type_tag(a);
 		const char *tag_b = drgn_type_tag(b);
@@ -363,12 +368,34 @@ struct drgn_type *drgn_void_type(struct drgn_program *prog,
 	return &prog->void_types[lang - drgn_languages];
 }
 
+static struct drgn_error *
+drgn_byte_order_to_little_endian(struct drgn_program *prog,
+				 enum drgn_byte_order byte_order, bool *ret)
+{
+	SWITCH_ENUM_DEFAULT(byte_order,
+	case DRGN_BIG_ENDIAN:
+		*ret = false;
+		return NULL;
+	case DRGN_LITTLE_ENDIAN:
+		*ret = true;
+		return NULL;
+	case DRGN_PROGRAM_ENDIAN:
+		return drgn_program_is_little_endian(prog, ret);
+	default:
+		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
+					 "invalid byte order");
+	)
+}
+
 struct drgn_error *drgn_int_type_create(struct drgn_program *prog,
 					const char *name, uint64_t size,
 					bool is_signed,
+					enum drgn_byte_order byte_order,
 					const struct drgn_language *lang,
 					struct drgn_type **ret)
 {
+	struct drgn_error *err;
+
 	enum drgn_primitive_type primitive = c_parse_specifier_list(name);
 	if (drgn_primitive_type_kind[primitive] == DRGN_TYPE_INT &&
 	    (primitive == DRGN_C_TYPE_CHAR ||
@@ -389,14 +416,21 @@ struct drgn_error *drgn_int_type_create(struct drgn_program *prog,
 			.language = lang ? lang : drgn_program_language(prog),
 		}
 	};
+	err = drgn_byte_order_to_little_endian(prog, byte_order,
+					       &key._private.little_endian);
+	if (err)
+		return err;
 	return find_or_create_type(&key, ret);
 }
 
 struct drgn_error *drgn_bool_type_create(struct drgn_program *prog,
 					 const char *name, uint64_t size,
+					 enum drgn_byte_order byte_order,
 					 const struct drgn_language *lang,
 					 struct drgn_type **ret)
 {
+	struct drgn_error *err;
+
 	enum drgn_primitive_type primitive = c_parse_specifier_list(name);
 	if (primitive == DRGN_C_TYPE_BOOL)
 		name = drgn_primitive_type_spellings[DRGN_C_TYPE_BOOL][0];
@@ -414,14 +448,21 @@ struct drgn_error *drgn_bool_type_create(struct drgn_program *prog,
 			.language = lang ? lang : drgn_program_language(prog),
 		}
 	};
+	err = drgn_byte_order_to_little_endian(prog, byte_order,
+					       &key._private.little_endian);
+	if (err)
+		return err;
 	return find_or_create_type(&key, ret);
 }
 
 struct drgn_error *drgn_float_type_create(struct drgn_program *prog,
 					  const char *name, uint64_t size,
+					  enum drgn_byte_order byte_order,
 					  const struct drgn_language *lang,
 					  struct drgn_type **ret)
 {
+	struct drgn_error *err;
+
 	enum drgn_primitive_type primitive = c_parse_specifier_list(name);
 	if (drgn_primitive_type_kind[primitive] == DRGN_TYPE_FLOAT)
 		name = drgn_primitive_type_spellings[primitive][0];
@@ -439,37 +480,10 @@ struct drgn_error *drgn_float_type_create(struct drgn_program *prog,
 			.language = lang ? lang : drgn_program_language(prog),
 		}
 	};
-	return find_or_create_type(&key, ret);
-}
-
-struct drgn_error *drgn_complex_type_create(struct drgn_program *prog,
-					    const char *name, uint64_t size,
-					    struct drgn_type *real_type,
-					    const struct drgn_language *lang,
-					    struct drgn_type **ret)
-{
-	if (drgn_type_program(real_type) != prog) {
-		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
-					 "type is from different program");
-	}
-	if (drgn_type_kind(real_type) != DRGN_TYPE_FLOAT &&
-	    drgn_type_kind(real_type) != DRGN_TYPE_INT) {
-		return drgn_error_create(DRGN_ERROR_TYPE,
-					 "real type of complex type must be floating-point or integer type");
-	}
-
-	struct drgn_type key = {
-		{
-			.kind = DRGN_TYPE_COMPLEX,
-			.is_complete = true,
-			.primitive = DRGN_NOT_PRIMITIVE_TYPE,
-			.name = name,
-			.size = size,
-			.type = real_type,
-			.program = prog,
-			.language = lang ? lang : drgn_program_language(prog),
-		}
-	};
+	err = drgn_byte_order_to_little_endian(prog, byte_order,
+					       &key._private.little_endian);
+	if (err)
+		return err;
 	return find_or_create_type(&key, ret);
 }
 
@@ -777,9 +791,12 @@ drgn_typedef_type_create(struct drgn_program *prog, const char *name,
 struct drgn_error *
 drgn_pointer_type_create(struct drgn_program *prog,
 			 struct drgn_qualified_type referenced_type,
-			 uint64_t size, const struct drgn_language *lang,
+			 uint64_t size, enum drgn_byte_order byte_order,
+			 const struct drgn_language *lang,
 			 struct drgn_type **ret)
 {
+	struct drgn_error *err;
+
 	if (drgn_type_program(referenced_type.type) != prog) {
 		return drgn_error_create(DRGN_ERROR_INVALID_ARGUMENT,
 					 "type is from different program");
@@ -797,6 +814,10 @@ drgn_pointer_type_create(struct drgn_program *prog,
 			.language = lang ? lang : drgn_program_language(prog),
 		}
 	};
+	err = drgn_byte_order_to_little_endian(prog, byte_order,
+					       &key._private.little_endian);
+	if (err)
+		return err;
 	return find_or_create_type(&key, ret);
 }
 
@@ -952,6 +973,127 @@ drgn_function_type_create(struct drgn_function_type_builder *builder,
 	return NULL;
 }
 
+static struct drgn_error *
+drgn_type_with_byte_order_impl(struct drgn_type **type,
+			       struct drgn_type **underlying_type,
+			       enum drgn_byte_order byte_order)
+{
+	struct drgn_error *err;
+	switch (drgn_type_kind(*type)) {
+	case DRGN_TYPE_INT:
+		err = drgn_int_type_create(drgn_type_program(*type),
+					   drgn_type_name(*type),
+					   drgn_type_size(*type),
+					   drgn_type_is_signed(*type),
+					   byte_order,
+					   drgn_type_language(*type), type);
+		if (!err)
+			*underlying_type = *type;
+		return err;
+	case DRGN_TYPE_BOOL:
+		err = drgn_bool_type_create(drgn_type_program(*type),
+					    drgn_type_name(*type),
+					    drgn_type_size(*type), byte_order,
+					    drgn_type_language(*type), type);
+		if (!err)
+			*underlying_type = *type;
+		return err;
+	case DRGN_TYPE_FLOAT:
+		err = drgn_float_type_create(drgn_type_program(*type),
+					     drgn_type_name(*type),
+					     drgn_type_size(*type), byte_order,
+					     drgn_type_language(*type), type);
+		if (!err)
+			*underlying_type = *type;
+		return err;
+	case DRGN_TYPE_POINTER:
+		err = drgn_pointer_type_create(drgn_type_program(*type),
+					       drgn_type_type(*type),
+					       drgn_type_size(*type),
+					       byte_order,
+					       drgn_type_language(*type), type);
+		if (!err)
+			*underlying_type = *type;
+		return err;
+	case DRGN_TYPE_TYPEDEF: {
+		struct drgn_qualified_type aliased_type = drgn_type_type(*type);
+		err = drgn_type_with_byte_order_impl(&aliased_type.type,
+						     underlying_type,
+						     byte_order);
+		if (err)
+			return err;
+		return drgn_typedef_type_create(drgn_type_program(*type),
+						drgn_type_name(*type),
+						aliased_type,
+						drgn_type_language(*type),
+						type);
+	}
+	case DRGN_TYPE_ENUM: {
+		assert(drgn_type_is_complete(*type));
+		struct drgn_type *compatible_type = drgn_type_type(*type).type;
+		struct drgn_type *unused;
+		err = drgn_type_with_byte_order_impl(&compatible_type, &unused,
+						     byte_order);
+		if (err)
+			return err;
+		struct drgn_enum_type_builder builder;
+		drgn_enum_type_builder_init(&builder,
+					    drgn_type_program(*type));
+		size_t num_enumerators =
+			drgn_type_num_enumerators(*type);
+		if (num_enumerators) {
+			if (!drgn_type_enumerator_vector_reserve(&builder.enumerators,
+								 num_enumerators)) {
+				drgn_enum_type_builder_deinit(&builder);
+				return &drgn_enomem;
+			}
+			memcpy(&builder.enumerators.data,
+			       drgn_type_enumerators(*type),
+			       num_enumerators * sizeof(struct drgn_type_enumerator));
+		}
+		err = drgn_enum_type_create(&builder, drgn_type_tag(*type),
+					    compatible_type,
+					    drgn_type_language(*type), type);
+		if (err) {
+			drgn_enum_type_builder_deinit(&builder);
+			return err;
+		}
+		*underlying_type = *type;
+		return err;
+	}
+	default:
+		return NULL;
+	}
+}
+
+struct drgn_error *
+drgn_type_with_byte_order(struct drgn_type **type,
+			  struct drgn_type **underlying_type,
+			  enum drgn_byte_order byte_order)
+{
+	struct drgn_error *err;
+	bool type_little_endian;
+	if (drgn_type_has_little_endian(*underlying_type)) {
+		type_little_endian =
+			drgn_type_little_endian(*underlying_type);
+	} else if (drgn_type_kind(*underlying_type) == DRGN_TYPE_ENUM &&
+		   drgn_type_is_complete(*underlying_type)) {
+		type_little_endian =
+			drgn_type_little_endian(drgn_type_type(*underlying_type).type);
+	} else {
+		return NULL;
+	}
+	bool little_endian;
+	err = drgn_byte_order_to_little_endian(drgn_type_program(*underlying_type),
+					       byte_order, &little_endian);
+	if (err)
+		return err;
+	if (type_little_endian == little_endian)
+		return NULL;
+	return drgn_type_with_byte_order_impl(type, underlying_type,
+					      drgn_byte_order_from_little_endian(little_endian));
+}
+
 LIBDRGN_PUBLIC struct drgn_error *
 drgn_format_type_name(struct drgn_qualified_type qualified_type, char **ret)
 {
@@ -1026,7 +1168,6 @@ LIBDRGN_PUBLIC struct drgn_error *drgn_type_sizeof(struct drgn_type *type,
 	case DRGN_TYPE_INT:
 	case DRGN_TYPE_BOOL:
 	case DRGN_TYPE_FLOAT:
-	case DRGN_TYPE_COMPLEX:
 	case DRGN_TYPE_POINTER:
 		*ret = drgn_type_size(type);
 		return NULL;
@@ -1082,8 +1223,6 @@ enum drgn_object_encoding drgn_type_object_encoding(struct drgn_type *type)
 		return DRGN_OBJECT_ENCODING_UNSIGNED;
 	case DRGN_TYPE_FLOAT:
 		return DRGN_OBJECT_ENCODING_FLOAT;
-	case DRGN_TYPE_COMPLEX:
-		return DRGN_OBJECT_ENCODING_BUFFER;
 	case DRGN_TYPE_STRUCT:
 	case DRGN_TYPE_UNION:
 	case DRGN_TYPE_CLASS:
@@ -1296,21 +1435,19 @@ default_size_t_or_ptrdiff_t(struct drgn_program *prog,
 		},
 	};
 	struct drgn_error *err;
-	uint8_t word_size;
-
-	err = drgn_program_word_size(prog, &word_size);
+	uint8_t address_size;
+	err = drgn_program_address_size(prog, &address_size);
 	if (err)
 		return err;
 	for (size_t i = 0; i < ARRAY_SIZE(integer_types[0]); i++) {
-		enum drgn_primitive_type integer_type;
+		enum drgn_primitive_type integer_type =
+			integer_types[type == DRGN_C_TYPE_PTRDIFF_T][i];
 		struct drgn_qualified_type qualified_type;
-
-		integer_type = integer_types[type == DRGN_C_TYPE_PTRDIFF_T][i];
 		err = drgn_program_find_primitive_type(prog, integer_type,
 						       &qualified_type.type);
 		if (err)
 			return err;
-		if (drgn_type_size(qualified_type.type) == word_size) {
+		if (drgn_type_size(qualified_type.type) == address_size) {
 			qualified_type.qualifiers = 0;
 			return drgn_typedef_type_create(prog,
 							drgn_primitive_type_spellings[type][0],
@@ -1362,63 +1499,75 @@ drgn_program_find_primitive_type(struct drgn_program *prog,
 	case DRGN_C_TYPE_CHAR:
 	case DRGN_C_TYPE_SIGNED_CHAR:
 		err = drgn_int_type_create(prog, spellings[0], 1, true,
+					   DRGN_PROGRAM_ENDIAN,
 					   &drgn_language_c, ret);
 		break;
 	case DRGN_C_TYPE_UNSIGNED_CHAR:
 		err = drgn_int_type_create(prog, spellings[0], 1, false,
+					   DRGN_PROGRAM_ENDIAN,
 					   &drgn_language_c, ret);
 		break;
 	case DRGN_C_TYPE_SHORT:
 		err = drgn_int_type_create(prog, spellings[0], 2, true,
+					   DRGN_PROGRAM_ENDIAN,
 					   &drgn_language_c, ret);
 		break;
 	case DRGN_C_TYPE_UNSIGNED_SHORT:
 		err = drgn_int_type_create(prog, spellings[0], 2, false,
+					   DRGN_PROGRAM_ENDIAN,
 					   &drgn_language_c, ret);
 		break;
 	case DRGN_C_TYPE_INT:
 		err = drgn_int_type_create(prog, spellings[0], 4, true,
+					   DRGN_PROGRAM_ENDIAN,
 					   &drgn_language_c, ret);
 		break;
 	case DRGN_C_TYPE_UNSIGNED_INT:
 		err = drgn_int_type_create(prog, spellings[0], 4, false,
+					   DRGN_PROGRAM_ENDIAN,
 					   &drgn_language_c, ret);
 		break;
 	/* long and unsigned long default to the word size. */
 	case DRGN_C_TYPE_LONG:
 	case DRGN_C_TYPE_UNSIGNED_LONG: {
-		uint8_t word_size;
-
-		err = drgn_program_word_size(prog, &word_size);
+		uint8_t address_size;
+		err = drgn_program_address_size(prog, &address_size);
 		if (err)
 			break;
-		err = drgn_int_type_create(prog, spellings[0], word_size,
+		err = drgn_int_type_create(prog, spellings[0], address_size,
 					   type == DRGN_C_TYPE_LONG,
+					   DRGN_PROGRAM_ENDIAN,
 					   &drgn_language_c, ret);
 		break;
 	}
 	case DRGN_C_TYPE_LONG_LONG:
 		err = drgn_int_type_create(prog, spellings[0], 8, true,
+					   DRGN_PROGRAM_ENDIAN,
 					   &drgn_language_c, ret);
 		break;
 	case DRGN_C_TYPE_UNSIGNED_LONG_LONG:
 		err = drgn_int_type_create(prog, spellings[0], 8, false,
+					   DRGN_PROGRAM_ENDIAN,
 					   &drgn_language_c, ret);
 		break;
 	case DRGN_C_TYPE_BOOL:
 		err = drgn_bool_type_create(prog, spellings[0], 1,
+					    DRGN_PROGRAM_ENDIAN,
 					    &drgn_language_c, ret);
 		break;
 	case DRGN_C_TYPE_FLOAT:
 		err = drgn_float_type_create(prog, spellings[0], 4,
+					     DRGN_PROGRAM_ENDIAN,
 					     &drgn_language_c, ret);
 		break;
 	case DRGN_C_TYPE_DOUBLE:
 		err = drgn_float_type_create(prog, spellings[0], 8,
+					     DRGN_PROGRAM_ENDIAN,
 					     &drgn_language_c, ret);
 		break;
 	case DRGN_C_TYPE_LONG_DOUBLE:
 		err = drgn_float_type_create(prog, spellings[0], 16,
+					     DRGN_PROGRAM_ENDIAN,
 					     &drgn_language_c, ret);
 		break;
 	case DRGN_C_TYPE_SIZE_T:
