@@ -1,5 +1,5 @@
 // Copyright (c) Facebook, Inc. and its affiliates.
-// SPDX-License-Identifier: GPL-3.0+
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 /**
  * @file
@@ -20,7 +20,7 @@
 #include <string.h>
 
 #ifndef LIBDRGN_PUBLIC
-#define LIBDRGN_PUBLIC __attribute__((visibility("default")))
+#define LIBDRGN_PUBLIC __attribute__((__visibility__("default")))
 #endif
 
 #ifdef NDEBUG
@@ -29,6 +29,8 @@
 #define UNREACHABLE() assert(!"reachable")
 #endif
 
+#define HOST_LITTLE_ENDIAN (__BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__)
+
 /**
  * Switch statement with an enum controlling expression that must have a case
  * for every enumeration value and a default case.
@@ -116,4 +118,26 @@ static inline void *malloc64(uint64_t size)
 	return malloc(size);
 }
 
+/** Return the maximum value of an @p n-byte unsigned integer. */
+static inline uint64_t uint_max(int n)
+{
+	assert(n >= 1 && n <= 8);
+	return UINT64_MAX >> (64 - 8 * n);
+}
+
+/**
+ * Safely add to a pointer which may be `NULL`.
+ *
+ * `NULL + 0` is undefined behavior, but it often arises naturally, like when
+ * computing the end of a dynamic array: `arr + length`. This works around the
+ * undefined behavior: `add_to_possibly_null_pointer(NULL, 0)` is defined as
+ * `NULL`.
+ *
+ * A more natural definition would be `i == 0 ? ptr : ptr + i`, but some
+ * versions of GCC and Clang generate an unnecessary branch or conditional move
+ * (https://gcc.gnu.org/bugzilla/show_bug.cgi?id=97225).
+ */
+#define add_to_possibly_null_pointer(ptr, i)	\
+	((typeof(ptr))((uintptr_t)(ptr) + (i) * sizeof(*(ptr))))
+
 #endif /* DRGN_UTIL_H */
