@@ -1,5 +1,5 @@
 # Copyright (c) Facebook, Inc. and its affiliates.
-# SPDX-License-Identifier: GPL-3.0+
+# SPDX-License-Identifier: GPL-3.0-or-later
 
 import functools
 import types
@@ -139,7 +139,6 @@ def identical(a, b):
                     "prog_",
                     "type_",
                     "address_",
-                    "byteorder_",
                     "bit_offset_",
                     "bit_field_size_",
                 ),
@@ -184,6 +183,7 @@ def identical(a, b):
                         "size",
                         "length",
                         "is_signed",
+                        "byteorder",
                         "type",
                         "members",
                         "enumerators",
